@@ -407,7 +407,6 @@ clutter_event_get_position (const ClutterEvent *event,
     case CLUTTER_STAGE_STATE:
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
-    case CLUTTER_DELETE:
     case CLUTTER_EVENT_LAST:
     case CLUTTER_PROXIMITY_IN:
     case CLUTTER_PROXIMITY_OUT:
@@ -415,11 +414,6 @@ clutter_event_get_position (const ClutterEvent *event,
     case CLUTTER_PAD_BUTTON_RELEASE:
     case CLUTTER_PAD_STRIP:
     case CLUTTER_PAD_RING:
-    case CLUTTER_IM_COMMIT:
-    case CLUTTER_IM_DELETE:
-    case CLUTTER_IM_PREEDIT:
-    case CLUTTER_DEVICE_ADDED:
-    case CLUTTER_DEVICE_REMOVED:
       graphene_point_init (position, 0.f, 0.f);
       break;
 
@@ -486,7 +480,6 @@ clutter_event_set_coords (ClutterEvent *event,
     case CLUTTER_STAGE_STATE:
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
-    case CLUTTER_DELETE:
     case CLUTTER_EVENT_LAST:
     case CLUTTER_PROXIMITY_IN:
     case CLUTTER_PROXIMITY_OUT:
@@ -494,11 +487,6 @@ clutter_event_set_coords (ClutterEvent *event,
     case CLUTTER_PAD_BUTTON_RELEASE:
     case CLUTTER_PAD_STRIP:
     case CLUTTER_PAD_RING:
-    case CLUTTER_IM_COMMIT:
-    case CLUTTER_IM_DELETE:
-    case CLUTTER_IM_PREEDIT:
-    case CLUTTER_DEVICE_ADDED:
-    case CLUTTER_DEVICE_REMOVED:
       break;
 
     case CLUTTER_ENTER:
@@ -1115,11 +1103,7 @@ clutter_event_set_device (ClutterEvent       *event,
     case CLUTTER_STAGE_STATE:
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
-    case CLUTTER_DELETE:
     case CLUTTER_EVENT_LAST:
-    case CLUTTER_IM_COMMIT:
-    case CLUTTER_IM_DELETE:
-    case CLUTTER_IM_PREEDIT:
       break;
 
     case CLUTTER_ENTER:
@@ -1174,11 +1158,6 @@ clutter_event_set_device (ClutterEvent       *event,
     case CLUTTER_PAD_RING:
       event->pad_ring.device = device;
       break;
-
-    case CLUTTER_DEVICE_ADDED:
-    case CLUTTER_DEVICE_REMOVED:
-      event->device.device = device;
-      break;
     }
 }
 
@@ -1220,10 +1199,6 @@ clutter_event_get_device (const ClutterEvent *event)
     case CLUTTER_STAGE_STATE:
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
-    case CLUTTER_DELETE:
-    case CLUTTER_IM_COMMIT:
-    case CLUTTER_IM_DELETE:
-    case CLUTTER_IM_PREEDIT:
     case CLUTTER_EVENT_LAST:
       break;
 
@@ -1279,11 +1254,6 @@ clutter_event_get_device (const ClutterEvent *event)
     case CLUTTER_PAD_RING:
       device = event->pad_ring.device;
       break;
-
-    case CLUTTER_DEVICE_ADDED:
-    case CLUTTER_DEVICE_REMOVED:
-      device = event->device.device;
-      break;
     }
 
   return device;
@@ -1437,16 +1407,6 @@ clutter_event_copy (const ClutterEvent *event)
                                           sizeof (gdouble) * n_axes);
       break;
 
-    case CLUTTER_IM_COMMIT:
-    case CLUTTER_IM_PREEDIT:
-      new_event->im.text = g_strdup (event->im.text);
-      break;
-
-    case CLUTTER_DEVICE_ADDED:
-    case CLUTTER_DEVICE_REMOVED:
-      new_event->device.device = event->device.device;
-      break;
-
     default:
       break;
     }
@@ -1502,11 +1462,6 @@ clutter_event_free (ClutterEvent *event)
           g_free (event->touch.axes);
           break;
 
-        case CLUTTER_IM_COMMIT:
-        case CLUTTER_IM_PREEDIT:
-          g_free (event->im.text);
-          break;
-
         default:
           break;
         }
@@ -1766,7 +1721,6 @@ clutter_event_get_axes (const ClutterEvent *event,
     case CLUTTER_STAGE_STATE:
     case CLUTTER_DESTROY_NOTIFY:
     case CLUTTER_CLIENT_MESSAGE:
-    case CLUTTER_DELETE:
     case CLUTTER_ENTER:
     case CLUTTER_LEAVE:
     case CLUTTER_KEY_PRESS:
@@ -1774,8 +1728,6 @@ clutter_event_get_axes (const ClutterEvent *event,
     case CLUTTER_EVENT_LAST:
     case CLUTTER_PROXIMITY_IN:
     case CLUTTER_PROXIMITY_OUT:
-    case CLUTTER_DEVICE_ADDED:
-    case CLUTTER_DEVICE_REMOVED:
       break;
 
     case CLUTTER_SCROLL:
@@ -1804,9 +1756,6 @@ clutter_event_get_axes (const ClutterEvent *event,
     case CLUTTER_PAD_BUTTON_RELEASE:
     case CLUTTER_PAD_STRIP:
     case CLUTTER_PAD_RING:
-    case CLUTTER_IM_COMMIT:
-    case CLUTTER_IM_DELETE:
-    case CLUTTER_IM_PREEDIT:
       break;
     }
 
