@@ -146,5 +146,27 @@ _cogl_pipeline_flush_gl_state (CoglContext *context,
                                gboolean skip_gl_state,
                                gboolean unknown_color_alpha);
 
+void
+_cogl_glsl_shader_set_source_with_boilerplate (CoglContext *ctx,
+                                               GLuint shader_gl_handle,
+                                               GLenum shader_gl_type,
+                                               CoglPipeline *pipeline,
+                                               GLsizei count_in,
+                                               const char **strings_in,
+                                               const GLint *lengths_in);
+
+void
+_cogl_sampler_gl_init (CoglContext *context,
+                       CoglSamplerCacheEntry *entry);
+
+void
+_cogl_sampler_gl_free (CoglContext *context,
+                       CoglSamplerCacheEntry *entry);
+
+void
+_cogl_gl_set_uniform (CoglContext *ctx,
+                      GLint location,
+                      const CoglBoxedValue *value);
+
 #endif /* __COGL_PIPELINE_OPENGL_PRIVATE_H */
 
