@@ -30,30 +30,12 @@ check_flags (TestFlags flags,
       return FALSE;
     }
 
-  if (flags & TEST_REQUIREMENT_POINT_SPRITE &&
-      !cogl_has_feature (test_ctx, COGL_FEATURE_ID_POINT_SPRITE))
-    {
-      return FALSE;
-    }
-
-  if (flags & TEST_REQUIREMENT_PER_VERTEX_POINT_SIZE &&
-      !cogl_has_feature (test_ctx, COGL_FEATURE_ID_PER_VERTEX_POINT_SIZE))
-    {
-      return FALSE;
-    }
-
   if (flags & TEST_REQUIREMENT_MAP_WRITE &&
       !cogl_has_feature (test_ctx, COGL_FEATURE_ID_MAP_BUFFER_FOR_WRITE))
     {
       return FALSE;
     }
 
-  if (flags & TEST_REQUIREMENT_OFFSCREEN &&
-      !cogl_has_feature (test_ctx, COGL_FEATURE_ID_OFFSCREEN))
-    {
-      return FALSE;
-    }
-
   if (flags & TEST_REQUIREMENT_FENCE &&
       !cogl_has_feature (test_ctx, COGL_FEATURE_ID_FENCE))
     {
@@ -95,7 +77,7 @@ is_boolean_env_set (const char *variable)
   return ret;
 }
 
-void
+gboolean
 test_utils_init (TestFlags requirement_flags,
                  TestFlags known_failure_flags)
 {
@@ -174,6 +156,8 @@ test_utils_init (TestFlags requirement_flags,
     g_print ("WARNING: Missing required feature[s] for this test\n");
   else if (known_failure)
     g_print ("WARNING: Test is known to fail\n");
+
+  return (!missing_requirement && !known_failure);
 }
 
 void
@@ -268,7 +252,17 @@ void
 test_utils_check_pixel_rgb (CoglFramebuffer *test_fb,
                             int x, int y, int r, int g, int b)
 {
-  test_utils_check_pixel (test_fb, x, y, (r << 24) | (g << 16) | (b << 8));
+  g_return_if_fail (r >= 0);
+  g_return_if_fail (g >= 0);
+  g_return_if_fail (b >= 0);
+  g_return_if_fail (r <= 0xFF);
+  g_return_if_fail (g <= 0xFF);
+  g_return_if_fail (b <= 0xFF);
+
+  test_utils_check_pixel (test_fb, x, y,
+                          (((guint32) r) << 24) |
+                          (((guint32) g) << 16) |
+                          (((guint32) b) << 8));
 }
 
 void
