@@ -26,6 +26,7 @@
 
 #include "backends/meta-backend-private.h"
 #include "backends/meta-logical-monitor.h"
+#include "backends/native/meta-crtc-mode-kms.h"
 #include "backends/native/meta-gpu-kms.h"
 #include "backends/native/meta-output-kms.h"
 #include "backends/native/meta-kms-device.h"
@@ -34,21 +35,38 @@
 
 #define ALL_TRANSFORMS_MASK ((1 << META_MONITOR_N_TRANSFORMS) - 1)
 
-typedef struct _MetaCrtcKms
+struct _MetaCrtcKms
 {
+  MetaCrtc parent;
+
   MetaKmsCrtc *kms_crtc;
 
   MetaKmsPlane *primary_plane;
-} MetaCrtcKms;
+
+  gpointer cursor_renderer_private;
+};
 
 static GQuark kms_crtc_crtc_kms_quark;
 
+G_DEFINE_TYPE (MetaCrtcKms, meta_crtc_kms, META_TYPE_CRTC)
+
+gpointer
+meta_crtc_kms_get_cursor_renderer_private (MetaCrtcKms *crtc_kms)
+{
+  return crtc_kms->cursor_renderer_private;
+}
+
+void
+meta_crtc_kms_set_cursor_renderer_private (MetaCrtcKms *crtc_kms,
+                                           gpointer     cursor_renderer_private)
+{
+  crtc_kms->cursor_renderer_private = cursor_renderer_private;
+}
+
 gboolean
-meta_crtc_kms_is_transform_handled (MetaCrtc             *crtc,
+meta_crtc_kms_is_transform_handled (MetaCrtcKms          *crtc_kms,
                                     MetaMonitorTransform  transform)
 {
-  MetaCrtcKms *crtc_kms = crtc->driver_private;
-
   if (!crtc_kms->primary_plane)
     return FALSE;
 
@@ -57,16 +75,19 @@ meta_crtc_kms_is_transform_handled (MetaCrtc             *crtc,
 }
 
 void
-meta_crtc_kms_apply_transform (MetaCrtc               *crtc,
+meta_crtc_kms_apply_transform (MetaCrtcKms            *crtc_kms,
                                MetaKmsPlaneAssignment *kms_plane_assignment)
 {
-  MetaCrtcKms *crtc_kms = crtc->driver_private;
+  MetaCrtc *crtc = META_CRTC (crtc_kms);
+  const MetaCrtcConfig *crtc_config;
   MetaMonitorTransform hw_transform;
 
-  hw_transform = crtc->transform;
-  if (!meta_crtc_kms_is_transform_handled (crtc, hw_transform))
+  crtc_config = meta_crtc_get_config (crtc);
+
+  hw_transform = crtc_config->transform;
+  if (!meta_crtc_kms_is_transform_handled (crtc_kms, hw_transform))
     hw_transform = META_MONITOR_TRANSFORM_NORMAL;
-  if (!meta_crtc_kms_is_transform_handled (crtc, hw_transform))
+  if (!meta_crtc_kms_is_transform_handled (crtc_kms, hw_transform))
     return;
 
   meta_kms_plane_update_set_rotation (crtc_kms->primary_plane,
@@ -75,12 +96,13 @@ meta_crtc_kms_apply_transform (MetaCrtc               *crtc,
 }
 
 void
-meta_crtc_kms_assign_primary_plane (MetaCrtc      *crtc,
+meta_crtc_kms_assign_primary_plane (MetaCrtcKms   *crtc_kms,
                                     uint32_t       fb_id,
                                     MetaKmsUpdate *kms_update)
 {
-  MetaRectangle logical_monitor_rect;
-  int x, y;
+  MetaCrtc *crtc = META_CRTC (crtc_kms);
+  const MetaCrtcConfig *crtc_config;
+  const MetaCrtcModeInfo *crtc_mode_info;
   MetaFixed16Rectangle src_rect;
   MetaFixed16Rectangle dst_rect;
   MetaKmsAssignPlaneFlag flags;
@@ -89,26 +111,25 @@ meta_crtc_kms_assign_primary_plane (MetaCrtc      *crtc,
   MetaKmsPlane *primary_kms_plane;
   MetaKmsPlaneAssignment *plane_assignment;
 
-  logical_monitor_rect =
-    meta_logical_monitor_get_layout (crtc->logical_monitor);
-  x = crtc->rect.x - logical_monitor_rect.x;
-  y = crtc->rect.y - logical_monitor_rect.y;
+  crtc_config = meta_crtc_get_config (crtc);
+  crtc_mode_info = meta_crtc_mode_get_info (crtc_config->mode);
+
   src_rect = (MetaFixed16Rectangle) {
-    .x = meta_fixed_16_from_int (x),
-    .y = meta_fixed_16_from_int (y),
-    .width = meta_fixed_16_from_int (crtc->rect.width),
-    .height = meta_fixed_16_from_int (crtc->rect.height),
+    .x = meta_fixed_16_from_int (0),
+    .y = meta_fixed_16_from_int (0),
+    .width = meta_fixed_16_from_int (crtc_mode_info->width),
+    .height = meta_fixed_16_from_int (crtc_mode_info->height),
   };
   dst_rect = (MetaFixed16Rectangle) {
     .x = meta_fixed_16_from_int (0),
     .y = meta_fixed_16_from_int (0),
-    .width = meta_fixed_16_from_int (crtc->rect.width),
-    .height = meta_fixed_16_from_int (crtc->rect.height),
+    .width = meta_fixed_16_from_int (crtc_mode_info->width),
+    .height = meta_fixed_16_from_int (crtc_mode_info->height),
   };
 
   flags = META_KMS_ASSIGN_PLANE_FLAG_NONE;
 
-  kms_crtc = meta_crtc_kms_get_kms_crtc (crtc);
+  kms_crtc = meta_crtc_kms_get_kms_crtc (crtc_kms);
   kms_device = meta_kms_crtc_get_device (kms_crtc);
   primary_kms_plane = meta_kms_device_get_primary_plane_for (kms_device,
                                                              kms_crtc);
@@ -119,7 +140,7 @@ meta_crtc_kms_assign_primary_plane (MetaCrtc      *crtc,
                                                    src_rect,
                                                    dst_rect,
                                                    flags);
-  meta_crtc_kms_apply_transform (crtc, plane_assignment);
+  meta_crtc_kms_apply_transform (crtc_kms, plane_assignment);
 }
 
 static GList *
@@ -138,7 +159,7 @@ generate_crtc_connector_list (MetaGpu  *gpu,
       if (assigned_crtc == crtc)
         {
           MetaKmsConnector *kms_connector =
-            meta_output_kms_get_kms_connector (output);
+            meta_output_kms_get_kms_connector (META_OUTPUT_KMS (output));
 
           connectors = g_list_prepend (connectors, kms_connector);
         }
@@ -148,57 +169,61 @@ generate_crtc_connector_list (MetaGpu  *gpu,
 }
 
 void
-meta_crtc_kms_set_mode (MetaCrtc      *crtc,
+meta_crtc_kms_set_mode (MetaCrtcKms   *crtc_kms,
                         MetaKmsUpdate *kms_update)
 {
+  MetaCrtc *crtc = META_CRTC (crtc_kms);
   MetaGpu *gpu = meta_crtc_get_gpu (crtc);
   GList *connectors;
-  drmModeModeInfo *mode;
+  const drmModeModeInfo *mode;
 
   connectors = generate_crtc_connector_list (gpu, crtc);
 
   if (connectors)
     {
-      mode = crtc->current_mode->driver_private;
+      const MetaCrtcConfig *crtc_config = meta_crtc_get_config (crtc);
+      MetaCrtcModeKms *crtc_mode_kms = META_CRTC_MODE_KMS (crtc_config->mode);
 
-      g_debug ("Setting CRTC (%ld) mode to %s", crtc->crtc_id, mode->name);
+      mode = meta_crtc_mode_kms_get_drm_mode (crtc_mode_kms);
+
+      g_debug ("Setting CRTC (%ld) mode to %s",
+               meta_crtc_get_id (crtc), mode->name);
     }
   else
     {
       mode = NULL;
 
-      g_debug ("Unsetting CRTC (%ld) mode", crtc->crtc_id);
+      g_debug ("Unsetting CRTC (%ld) mode",
+               meta_crtc_get_id (crtc));
     }
 
   meta_kms_update_mode_set (kms_update,
-                            meta_crtc_kms_get_kms_crtc (crtc),
+                            meta_crtc_kms_get_kms_crtc (crtc_kms),
                             g_steal_pointer (&connectors),
                             mode);
 }
 
 void
-meta_crtc_kms_page_flip (MetaCrtc                      *crtc,
+meta_crtc_kms_page_flip (MetaCrtcKms                   *crtc_kms,
                          const MetaKmsPageFlipFeedback *page_flip_feedback,
                          gpointer                       user_data,
                          MetaKmsUpdate                 *kms_update)
 {
   meta_kms_update_page_flip (kms_update,
-                             meta_crtc_kms_get_kms_crtc (crtc),
+                             meta_crtc_kms_get_kms_crtc (crtc_kms),
                              page_flip_feedback,
                              user_data);
 }
 
 MetaKmsCrtc *
-meta_crtc_kms_get_kms_crtc (MetaCrtc *crtc)
+meta_crtc_kms_get_kms_crtc (MetaCrtcKms *crtc_kms)
 {
-  MetaCrtcKms *crtc_kms = crtc->driver_private;
-
   return crtc_kms->kms_crtc;
 }
 
 /**
  * meta_crtc_kms_get_modifiers:
- * @crtc: a #MetaCrtc object that has to be a #MetaCrtcKms
+ * @crtc_kms: a #MetaCrtc object that has to be a #MetaCrtcKms
  * @format: a DRM pixel format
  *
  * Returns a pointer to a #GArray containing all the supported
@@ -211,114 +236,85 @@ meta_crtc_kms_get_kms_crtc (MetaCrtc *crtc)
  * supported.
  */
 GArray *
-meta_crtc_kms_get_modifiers (MetaCrtc *crtc,
-                             uint32_t  format)
+meta_crtc_kms_get_modifiers (MetaCrtcKms *crtc_kms,
+                             uint32_t     format)
 {
-  MetaCrtcKms *crtc_kms = crtc->driver_private;
-
   return meta_kms_plane_get_modifiers_for_format (crtc_kms->primary_plane,
                                                   format);
 }
 
 /**
  * meta_crtc_kms_copy_drm_format_list:
- * @crtc: a #MetaCrtc object that has to be a #MetaCrtcKms
+ * @crtc_kms: a #MetaCrtc object that has to be a #MetaCrtcKms
  *
  * Returns a new #GArray that the caller must destroy. The array
  * contains all the DRM pixel formats the CRTC supports on
  * its primary plane. The array element type is uint32_t.
  */
 GArray *
-meta_crtc_kms_copy_drm_format_list (MetaCrtc *crtc)
+meta_crtc_kms_copy_drm_format_list (MetaCrtcKms *crtc_kms)
 {
-  MetaCrtcKms *crtc_kms = crtc->driver_private;
-
   return meta_kms_plane_copy_drm_format_list (crtc_kms->primary_plane);
 }
 
 /**
  * meta_crtc_kms_supports_format:
- * @crtc: a #MetaCrtc object that has to be a #MetaCrtcKms
+ * @crtc_kms: a #MetaCrtcKms
  * @drm_format: a DRM pixel format
  *
  * Returns true if the CRTC supports the format on its primary plane.
  */
 gboolean
-meta_crtc_kms_supports_format (MetaCrtc *crtc,
-                               uint32_t  drm_format)
+meta_crtc_kms_supports_format (MetaCrtcKms *crtc_kms,
+                               uint32_t     drm_format)
 {
-  MetaCrtcKms *crtc_kms = crtc->driver_private;
-
   return meta_kms_plane_is_format_supported (crtc_kms->primary_plane,
                                              drm_format);
 }
 
-MetaCrtc *
+MetaCrtcKms *
 meta_crtc_kms_from_kms_crtc (MetaKmsCrtc *kms_crtc)
 {
   return g_object_get_qdata (G_OBJECT (kms_crtc), kms_crtc_crtc_kms_quark);
 }
 
-static void
-meta_crtc_destroy_notify (MetaCrtc *crtc)
-{
-  g_free (crtc->driver_private);
-}
-
-MetaCrtc *
-meta_create_kms_crtc (MetaGpuKms  *gpu_kms,
-                      MetaKmsCrtc *kms_crtc)
+MetaCrtcKms *
+meta_crtc_kms_new (MetaGpuKms  *gpu_kms,
+                   MetaKmsCrtc *kms_crtc)
 {
   MetaGpu *gpu = META_GPU (gpu_kms);
   MetaKmsDevice *kms_device;
-  MetaCrtc *crtc;
   MetaCrtcKms *crtc_kms;
   MetaKmsPlane *primary_plane;
-  const MetaKmsCrtcState *crtc_state;
 
   kms_device = meta_gpu_kms_get_kms_device (gpu_kms);
   primary_plane = meta_kms_device_get_primary_plane_for (kms_device,
                                                          kms_crtc);
-  crtc_state = meta_kms_crtc_get_current_state (kms_crtc);
+  crtc_kms = g_object_new (META_TYPE_CRTC_KMS,
+                           "id", (uint64_t) meta_kms_crtc_get_id (kms_crtc),
+                           "gpu", gpu,
+                           NULL);
 
-  crtc = g_object_new (META_TYPE_CRTC, NULL);
-  crtc->gpu = gpu;
-  crtc->crtc_id = meta_kms_crtc_get_id (kms_crtc);
-  crtc->rect = crtc_state->rect;
-  crtc->is_dirty = FALSE;
-  crtc->transform = META_MONITOR_TRANSFORM_NORMAL;
-  crtc->all_transforms = ALL_TRANSFORMS_MASK;
-
-  if (crtc_state->is_drm_mode_valid)
-    {
-      GList *l;
-
-      for (l = meta_gpu_get_modes (gpu); l; l = l->next)
-        {
-          MetaCrtcMode *mode = l->data;
-
-          if (meta_drm_mode_equal (&crtc_state->drm_mode, mode->driver_private))
-            {
-              crtc->current_mode = mode;
-              break;
-            }
-        }
-    }
-
-  crtc_kms = g_new0 (MetaCrtcKms, 1);
   crtc_kms->kms_crtc = kms_crtc;
   crtc_kms->primary_plane = primary_plane;
 
-  crtc->driver_private = crtc_kms;
-  crtc->driver_notify = (GDestroyNotify) meta_crtc_destroy_notify;
-
   if (!kms_crtc_crtc_kms_quark)
     {
       kms_crtc_crtc_kms_quark =
         g_quark_from_static_string ("meta-kms-crtc-crtc-kms-quark");
     }
 
-  g_object_set_qdata (G_OBJECT (kms_crtc), kms_crtc_crtc_kms_quark, crtc);
+  g_object_set_qdata (G_OBJECT (kms_crtc), kms_crtc_crtc_kms_quark, crtc_kms);
 
-  return crtc;
+  return crtc_kms;
+}
+
+static void
+meta_crtc_kms_init (MetaCrtcKms *crtc_kms)
+{
+}
+
+static void
+meta_crtc_kms_class_init (MetaCrtcKmsClass *klass)
+{
 }
