@@ -48,9 +48,6 @@ gboolean meta_gpu_kms_is_crtc_active (MetaGpuKms *gpu_kms,
 gboolean meta_gpu_kms_is_boot_vga (MetaGpuKms *gpu_kms);
 gboolean meta_gpu_kms_is_platform_device (MetaGpuKms *gpu_kms);
 
-gboolean meta_gpu_kms_wait_for_flip (MetaGpuKms *gpu_kms,
-                                     GError    **error);
-
 MetaKmsDevice * meta_gpu_kms_get_kms_device (MetaGpuKms *gpu_kms);
 
 int meta_gpu_kms_get_fd (MetaGpuKms *gpu_kms);
@@ -77,4 +74,21 @@ MetaGpuKmsFlipClosureContainer * meta_gpu_kms_wrap_flip_closure (MetaGpuKms *gpu
 
 void meta_gpu_kms_flip_closure_container_free (MetaGpuKmsFlipClosureContainer *closure_container);
 
+typedef struct _MetaGpuKmsFBArgs
+{
+  uint32_t width;
+  uint32_t height;
+  uint32_t format;
+  uint32_t handles[4];
+  uint32_t offsets[4];
+  uint32_t strides[4];
+  uint64_t modifiers[4];
+} MetaGpuKmsFBArgs;
+
+gboolean meta_gpu_kms_add_fb (MetaGpuKms              *gpu_kms,
+                              gboolean                 use_modifiers,
+                              const MetaGpuKmsFBArgs  *args,
+                              uint32_t                *fb_id_out,
+                              GError                 **error);
+
 #endif /* META_GPU_KMS_H */
