@@ -35,6 +35,10 @@ const char * meta_kms_device_get_path (MetaKmsDevice *device);
 
 MetaKmsDeviceFlag meta_kms_device_get_flags (MetaKmsDevice *device);
 
+gboolean meta_kms_device_get_cursor_size (MetaKmsDevice *device,
+                                          uint64_t      *out_cursor_width,
+                                          uint64_t      *out_cursor_height);
+
 GList * meta_kms_device_get_connectors (MetaKmsDevice *device);
 
 GList * meta_kms_device_get_crtcs (MetaKmsDevice *device);
@@ -42,8 +46,8 @@ GList * meta_kms_device_get_crtcs (MetaKmsDevice *device);
 MetaKmsPlane * meta_kms_device_get_primary_plane_for (MetaKmsDevice *device,
                                                       MetaKmsCrtc   *crtc);
 
-int meta_kms_device_dispatch_sync (MetaKmsDevice  *device,
-                                   GError        **error);
+MetaKmsPlane * meta_kms_device_get_cursor_plane_for (MetaKmsDevice *device,
+                                                     MetaKmsCrtc   *crtc);
 
 MetaKmsDevice * meta_kms_device_new (MetaKms            *kms,
                                      const char         *path,
