@@ -62,6 +62,7 @@ enum
 
   PROP_N_WINDOWS,
   PROP_WORKSPACE_INDEX,
+  PROP_ACTIVE,
 
   PROP_LAST,
 };
@@ -179,6 +180,9 @@ meta_workspace_get_property (GObject      *object,
     case PROP_WORKSPACE_INDEX:
       g_value_set_uint (value, meta_workspace_index (ws));
       break;
+    case PROP_ACTIVE:
+      g_value_set_boolean (value, ws->manager->active_workspace == ws);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -218,6 +222,11 @@ meta_workspace_class_init (MetaWorkspaceClass *klass)
                                                        "The workspace's index",
                                                        0, G_MAXUINT, 0,
                                                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  obj_props[PROP_ACTIVE] = g_param_spec_boolean ("active",
+                                                 "Active",
+                                                 "Whether the workspace is currently active",
+                                                 FALSE,
+                                                 G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   g_object_class_install_properties (object_class, PROP_LAST, obj_props);
 }
@@ -551,9 +560,13 @@ meta_workspace_activate_with_focus (MetaWorkspace *workspace,
 
   g_signal_emit_by_name (workspace->manager, "active-workspace-changed");
 
+  g_object_notify_by_pspec (G_OBJECT (workspace), obj_props[PROP_ACTIVE]);
+
   if (old == NULL)
     return;
 
+  g_object_notify_by_pspec (G_OBJECT (old), obj_props[PROP_ACTIVE]);
+
   /* If the "show desktop" mode is active for either the old workspace
    * or the new one *but not both*, then update the
    * _net_showing_desktop hint
@@ -1332,10 +1345,7 @@ find_focusable_ancestor (MetaWindow *window,
 {
   MetaWorkspaceFocusableAncestorData *data = user_data;
 
-  if (!window->unmanaging &&
-      window->mapped &&
-      !window->hidden &&
-      meta_window_is_focusable (window) &&
+  if (!window->unmanaging && meta_window_is_focusable (window) &&
       meta_window_located_on_workspace (window, data->workspace) &&
       meta_window_showing_on_its_workspace (window))
     {
@@ -1346,38 +1356,6 @@ find_focusable_ancestor (MetaWindow *window,
   return TRUE;
 }
 
-static gboolean
-try_to_set_focus_and_check (MetaWindow *window,
-                            MetaWindow *not_this_one,
-                            uint32_t    timestamp)
-{
-  meta_window_focus (window, timestamp);
-
-  /* meta_focus_window() will not change focus for clients using the
-   * "globally active input" model of input handling, hence defeating
-   * the assumption that focus should be changed for such windows.
-   * See https://tronche.com/gui/x/icccm/sec-4.html#s-4.1.7
-   */
-  if (meta_window_is_focus_async (window))
-    return TRUE;
-
-  /* meta_window_focus() does not guarantee that focus will end up
-   * where we expect, it can fail for various reasons, better check
-   * it did not actually changed or even left focus to the window we
-   * explicitly want to avoid.
-   */
-  if (not_this_one &&
-      meta_display_get_focus_window (window->display) == not_this_one)
-    {
-      meta_warning ("Failed to focus window %s while avoiding %s",
-                    window->desc, not_this_one->desc);
-
-      return FALSE;
-    }
-
-  return TRUE;
-}
-
 /* Focus ancestor of not_this_one if there is one */
 static void
 focus_ancestor_or_top_window (MetaWorkspace *workspace,
@@ -1411,14 +1389,13 @@ focus_ancestor_or_top_window (MetaWorkspace *workspace,
                       "Focusing %s, ancestor of %s\n",
                       ancestor->desc, not_this_one->desc);
 
-          if (try_to_set_focus_and_check (ancestor, not_this_one, timestamp))
-            {
-              /* Also raise the window if in click-to-focus */
-              if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_CLICK)
-                meta_window_raise (ancestor);
+          meta_window_focus (ancestor, timestamp);
 
-              return;
-            }
+          /* Also raise the window if in click-to-focus */
+          if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_CLICK)
+            meta_window_raise (ancestor);
+
+          return;
         }
     }
 
@@ -1430,19 +1407,18 @@ focus_ancestor_or_top_window (MetaWorkspace *workspace,
     {
       meta_topic (META_DEBUG_FOCUS,
                   "Focusing workspace MRU window %s\n", window->desc);
-      if (try_to_set_focus_and_check (window, not_this_one, timestamp))
-        {
-          /* Also raise the window if in click-to-focus */
-          if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_CLICK)
-            meta_window_raise (window);
 
-          return;
-        }
+      meta_window_focus (window, timestamp);
+
+      /* Also raise the window if in click-to-focus */
+      if (meta_prefs_get_focus_mode () == G_DESKTOP_FOCUS_MODE_CLICK)
+        meta_window_raise (window);
+    }
+  else
+    {
+      meta_topic (META_DEBUG_FOCUS, "No MRU window to focus found; focusing no_focus_window.\n");
+      meta_display_unset_input_focus (workspace->display, timestamp);
     }
-
-  meta_topic (META_DEBUG_FOCUS,
-             "No MRU window to focus found; focusing no_focus_window.");
-  meta_display_unset_input_focus (workspace->display, timestamp);
 }
 
 /**
